@@ -1,5 +1,4 @@
-# vim: set ts=4 sw=4 expandtab sts=4 fileencoding=utf-8:
-# Copyright (c) 2011-2014 Christian Geier et al.
+# Copyright (c) 2013-2016 Christian Geier et al.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -22,13 +21,22 @@
 
 
 class Error(Exception):
+
     """base class for all of khal's Exceptions"""
     pass
 
+
 class FatalError(Error):
+
     """execution cannot continue"""
     pass
 
+
 class UnsupportedFeatureError(Error):
+
     """something Failed but we know why"""
     pass
+
+
+class InvalidDate(Error):
+    pass
