@@ -1,5 +1,4 @@
-# vim: set ts=4 sw=4 expandtab sts=4 fileencoding=utf-8:
-# Copyright (c) 2013-2014 Christian Geier et al.
+# Copyright (c) 2013-2016 Christian Geier et al.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -20,4 +19,4 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from .khalendar import Calendar, CalendarCollection  # flake8: noqa
+from .khalendar import CalendarCollection  # flake8: noqa
