@@ -1,5 +1,4 @@
-# vim: set ts=4 sw=4 expandtab sts=4 fileencoding=utf-8:
-# Copyright (c) 2011-2014 Christian Geier et al.
+# Copyright (c) 2013-2016 Christian Geier et al.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -20,11 +19,13 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-from ..exceptions import UnsupportedFeatureError
+from ..exceptions import UnsupportedFeatureError, Error, FatalError
 
 
 class UnsupportedRruleExceptionError(UnsupportedFeatureError):
+
     """we do not support exceptions that do not delete events yet"""
+
     def __init__(self, message=''):
         x = 'This kind of recurrence exception is currently unsupported'
         if message:
@@ -32,5 +33,34 @@ class UnsupportedRruleExceptionError(UnsupportedFeatureError):
         UnsupportedFeatureError.__init__(self, x)
 
 
-class ReadOnlyCalendarError(Exception):
-    """this calendar is readonly and should not be modifiable from within khal"""
+class ReadOnlyCalendarError(Error):
+
+    """this calendar is readonly and should not be modifiable from within
+    khal"""
+
+
+class OutdatedDbVersionError(FatalError):
+
+    """the db file has an older version and needs to be deleted"""
+
+
+class CouldNotCreateDbDir(FatalError):
+
+    """the db directory could not be created. Abort."""
+
+
+class UpdateFailed(Error):
+
+    """could not update the event in the database"""
+
+
+class UnsupportedRecursion(Error):
+
+    """raised if the RRULE is not understood by dateutil.rrule"""
+    pass
+
+
+class DuplicateUid(Error):
+
+    """an event with this UID already exists"""
+    existing_href = None
