@@ -1,5 +1,4 @@
-# vim: set ts=4 sw=4 expandtab sts=4 fileencoding=utf-8:
-# Copyright (c) 2011-2014 Christian Geier et al.
+# Copyright (c) 2013-2016 Christian Geier et al.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -23,9 +22,41 @@
 import logging
 import sys
 
+import click
+
 from khal import __productname__
 
-stdout_handler = logging.StreamHandler(sys.stdout)
+
+class ColorFormatter(logging.Formatter):
+    colors = {
+        'error': dict(fg='red'),
+        'exception': dict(fg='red'),
+        'critical': dict(fg='red'),
+        'debug': dict(fg='blue'),
+        'warning': dict(fg='yellow')
+    }
+
+    def format(self, record):
+        if not record.exc_info:
+            level = record.levelname.lower()
+            if level in self.colors:
+                prefix = click.style('{}: '.format(level),
+                                     **self.colors[level])
+                record.msg = '\n'.join(prefix + x
+                                       for x in str(record.msg).splitlines())
+
+        return logging.Formatter.format(self, record)
+
+
+class ClickStream(object):
+
+    def write(self, string):
+        click.echo(string, file=sys.stderr, nl=False)
+
+
+stdout_handler = logging.StreamHandler(ClickStream())
+stdout_handler.formatter = ColorFormatter()
+
 logger = logging.getLogger(__productname__)
 logger.setLevel(logging.INFO)
 logger.addHandler(stdout_handler)
