@@ -1,5 +1,4 @@
-# vim: set ts=4 sw=4 expandtab sts=4 fileencoding=utf-8:
-# Copyright (c) 2011-2014 Christian Geier et al.
+# Copyright (c) 2013-2016 Christian Geier et al.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -21,47 +20,13 @@
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 
+"""this module should contain classes that are specific to ikhal, more
+general widgets should go in widgets.py"""
+
+
 import urwid
-
-
-def vimify(key):
-    if key == 'h':
-        return 'left'
-    elif key == 'j':
-        return 'down'
-    elif key == 'k':
-        return 'up'
-    elif key == 'l':
-        return 'right'
-    # not really sure if these last to make any sense (not yet at least)
-    # at least for the time being, they are more trouble, than they are worth
-    # elif key == '0':
-        # return 'home'
-    # elif key == '$':
-        # return 'end'
-    else:
-        return key
-
-
-class CColumns(urwid.Columns):
-
-    def keypress(self, size, key):
-        # key = vimify(key)
-        return urwid.Columns.keypress(self, size, key)
-
-
-class CPile(urwid.Pile):
-
-    def keypress(self, size, key):
-        # key = vimify(key)
-        return urwid.Pile.keypress(self, size, key)
-
-
-class CSimpleFocusListWalker(urwid.SimpleFocusListWalker):
-
-    def keypress(self, size, key):
-        # key = vimify(key)
-        return urwid.SimpleFocusListWalker.keypress(self, size, key)
+import threading
+import time
 
 
 class Pane(urwid.WidgetWrap):
@@ -123,7 +88,6 @@ class HelpPane(Pane):
 
 
 class Window(urwid.Frame):
-
     """The main user interface frame.
 
     A window is a frame which displays a header, a footer and a body.
@@ -135,57 +99,22 @@ class Window(urwid.Frame):
     provides a mechanism allowing the panes to chain themselves, and
     to carry data between them.
     """
-    PALETTE = [('header', 'white', 'black'),
-               ('footer', 'white', 'black'),
-               ('line header', 'black', 'white', 'bold'),
-               ('bright', 'dark blue', 'white', ('bold', 'standout')),
-               ('list', 'black', 'white'),
-               ('list focused', 'white', 'light blue', 'bold'),
-               ('edit', 'black', 'white'),
-               ('edit focused', 'white', 'light blue', 'bold'),
-               ('button', 'black', 'dark cyan'),
-               ('button focused', 'white', 'light blue', 'bold'),
-               ('reveal focus', 'black', 'dark cyan', 'standout'),
-               ('today_focus', 'white', 'black', 'standout'),
-               ('today', 'black', 'white', 'dark cyan'),
-               ('edit', 'white', 'dark blue'),
-               ('alert', 'white', 'dark red'),
 
-               ('editfc', 'white', 'dark blue', 'bold'),
-               ('editbx', 'light gray', 'dark blue'),
-               ('editcp', 'black', 'light gray', 'standout'),
-               ('popupbg', 'white', 'black', 'bold'),
-
-               ('black', 'black', ''),
-               ('dark red', 'dark red', ''),
-               ('dark green', 'dark green', ''),
-               ('brown', 'brown', ''),
-               ('dark blue', 'dark blue', ''),
-               ('dark magenta', 'dark magenta', ''),
-               ('dark cyan', 'dark cyan', ''),
-               ('light gray', 'light gray', ''),
-               ('dark gray', 'dark gray', ''),
-               ('light red', 'light red', ''),
-               ('light green', 'light green', ''),
-               ('yellow', 'yellow', ''),
-               ('light blue', 'light blue', ''),
-               ('light magenta', 'light magenta', ''),
-               ('light cyan', 'light cyan', ''),
-               ('white', 'white', ''),
-               ]
-
-    def __init__(self, header='', footer=''):
+    def __init__(self, footer=''):
         self._track = []
-        self._title = header
-        self._footer = footer
 
-        header = urwid.AttrWrap(urwid.Text(self._title), 'header')
-        footer = urwid.AttrWrap(urwid.Text(self._footer), 'footer')
+        header = urwid.AttrWrap(urwid.Text(''), 'header')
+        footer = urwid.AttrWrap(urwid.Text(footer), 'footer')
         urwid.Frame.__init__(self, urwid.Text(''),
                              header=header,
                              footer=footer)
+        self.update_header()
         self._original_w = None
 
+        self._alert_daemon = AlertDaemon(self.update_header)
+        self._alert_daemon.start()
+        self.alert = self._alert_daemon.alert
+
     def open(self, pane, callback=None):
         """Open a new pane.
 
@@ -197,15 +126,6 @@ class Window(urwid.Frame):
         self._track.append((pane, callback))
         self._update(pane)
 
-    def overlay(self, overlay_w, title):
-        """put overlay_w as an overlay over the currently active pane
-        """
-        overlay = Pane(urwid.Overlay(urwid.Filler(overlay_w),
-                                     self._get_current_pane(),
-                                     'center', 60,
-                                     'middle', 5), title)
-        self.open(overlay)
-
     def backtrack(self, data=None):
         """Unstack the displayed pane.
 
@@ -214,7 +134,7 @@ class Window(urwid.Frame):
         this callback is called with the given data (if any) before
         the previous pane gets redrawn.
         """
-        _, cb = self._track.pop()
+        old_pane, cb = self._track.pop()
         if cb:
             cb(data)
 
@@ -231,8 +151,49 @@ class Window(urwid.Frame):
             self.open(HelpPane(self._get_current_pane()))
 
     def _update(self, pane):
-        self.header.w.set_text(u'%s | %s' % (self._title, pane.title))
         self.set_body(pane)
+        self.update_header()
 
     def _get_current_pane(self):
         return self._track[-1][0] if self._track else None
+
+    def update_header(self, alert=None):
+        pane_title = getattr(self._get_current_pane(), 'title', None)
+        text = []
+
+        for part in (pane_title, alert):
+            if part:
+                text.append(part)
+                text.append(('black', ' | '))
+
+        self.header.w.set_text(text[:-1] or '')
+
+
+class AlertDaemon(threading.Thread):
+    def __init__(self, set_msg_func):
+        threading.Thread.__init__(self)
+        self._set_msg_func = set_msg_func
+        self.daemon = True
+        self._start_countdown = threading.Event()
+
+    def alert(self, msg):
+        self._set_msg_func(msg)
+        self._start_countdown.set()
+
+    def run(self):
+        # This is a daemon thread. Since the global namespace is going to
+        # vanish on interpreter shutdown, redefine everything from the global
+        # namespace here.
+        _sleep = time.sleep
+        _exception = Exception
+        _event = self._start_countdown
+        _set_msg = self._set_msg_func
+
+        while True:
+            _event.wait()
+            _sleep(3)
+            try:
+                _set_msg(None)
+            except _exception:
+                pass
+            _event.clear()
